#!/bin/sh

# Copyright (c) 2008-2011 MechSoft Mechanical and Software Solutions
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# -----------------------------------------------------------------------------
# Start/Stop Script for the SIWPAS Server
#
# Environment Variable Prerequisites
#
#   SIWPAS_HOME   May point at your Siwpas "build" directory.
#
#   SIWPAS_BASE   (Optional) Base directory for resolving dynamic portions
#                   of a Siwpas installation.  If not present, resolves to
#                   the same directory that SIWPAS_HOME points to.
#
#   SIWPAS_OUT    (Optional) Full path to a file where stdout and stderr
#                   will be redirected.
#                   Default is $SIWPAS_BASE/logs/siwpas.out
#
#   SIWPAS_OPTS   (Optional) Java runtime options used when the "start",
#                   or "run" command is executed.
#
#   SIWPAS_TMPDIR (Optional) Directory path location of temporary directory
#                   the JVM should use (java.io.tmpdir).  Defaults to
#                   $SIWPAS_BASE/temp.
#
#   JAVA_HOME       Must point at your Java Development Kit installation.
#                   Required to run the with the "debug" argument.
#
#   JRE_HOME        Must point at your Java Development Kit installation.
#                   Defaults to JAVA_HOME if empty.
#
#   JAVA_OPTS       (Optional) Java runtime options used when the "start",
#                   "stop", or "run" command is executed.
#
#   JAVA_ENDORSED_DIRS (Optional) Lists of of colon separated directories
#                   containing some jars in order to allow replacement of APIs
#                   created outside of the JCP (i.e. DOM and SAX from W3C).
#                   It can also be used to update the XML parser implementation.
#                   Defaults to $SIWPAS_HOME/endorsed.
#
#   JPDA_TRANSPORT  (Optional) JPDA transport used when the "jpda start"
#                   command is executed. The default is "dt_socket".
#
#   JPDA_ADDRESS    (Optional) Java runtime options used when the "jpda start"
#                   command is executed. The default is 8000.
#
#   JPDA_SUSPEND    (Optional) Java runtime options used when the "jpda start"
#                   command is executed. Specifies whether JVM should suspend
#                   execution immediately after startup. Default is "n".
#
#   JPDA_OPTS       (Optional) Java runtime options used when the "jpda start"
#                   command is executed. If used, JPDA_TRANSPORT, JPDA_ADDRESS,
#                   and JPDA_SUSPEND are ignored. Thus, all required jpda
#                   options MUST be specified. The default is:
#
#                   -agentlib:jdwp=transport=$JPDA_TRANSPORT,
#                       address=$JPDA_ADDRESS,server=y,suspend=$JPDA_SUSPEND
#
#   SIWPAS_PID    (Optional) Path of the file which should contains the pid
#                   of the siwpas startup java process, when start (fork) is
#                   used
#
#   LOGGING_CONFIG  (Optional) Override Siwpas's logging config file
#                   Example (all one line)
#                   LOGGING_CONFIG="-Djava.util.logging.config.file=$SIWPAS_BASE/conf/logging.properties"
#
#   JAAS_CONFIG  (Optional) Override Siwpas's JAAS config file
#                   Example (all one line)
#                   JAAS_CONFIG="-Djava.security.auth.login.config=$SIWPAS_BASE/conf/login.config"
#
#   LOGGING_MANAGER (Optional) Override Siwpas's logging manager
#                   Example (all one line)
#                   LOGGING_MANAGER="-Djava.util.logging.manager=org.apache.juli.ClassLoaderLogManager"
#
# -----------------------------------------------------------------------------

# OS specific support.  $var _must_ be set to either true or false.
cygwin=false
os400=false
darwin=false
case "`uname`" in
CYGWIN*) cygwin=true;;
OS400*) os400=true;;
Darwin*) darwin=true;;
esac

# resolve links - $0 may be a softlink
PRG="$0"

while [ -h "$PRG" ]; do
  ls=`ls -ld "$PRG"`
  link=`expr "$ls" : '.*-> \(.*\)$'`
  if expr "$link" : '/.*' > /dev/null; then
    PRG="$link"
  else
    PRG=`dirname "$PRG"`/"$link"
  fi
done

# Get standard environment variables
PRGDIR=`dirname "$PRG"`

# Only set SIWPAS_HOME if not already set
[ -z "$SIWPAS_HOME" ] && SIWPAS_HOME=`cd "$PRGDIR/.." >/dev/null; pwd`

# Ensure that any user defined CLASSPATH variables are not used on startup,
# but allow them to be specified in setenv.sh, in rare case when it is needed.
CLASSPATH=

SETENVPATH="${SIWPAS_BASE:-$SIWPAS_HOME}"
if [ -r "$SETENVPATH/bin/setenv.sh" ]; then
  . "$SETENVPATH/bin/setenv.sh"
elif [ -r "$SIWPAS_HOME/bin/setenv.sh" ]; then
  . "$SIWPAS_HOME/bin/setenv.sh"
fi

# For Cygwin, ensure paths are in UNIX format before anything is touched
if $cygwin; then
  [ -n "$JAVA_HOME" ] && JAVA_HOME=`cygpath --unix "$JAVA_HOME"`
  [ -n "$JRE_HOME" ] && JRE_HOME=`cygpath --unix "$JRE_HOME"`
  [ -n "$SIWPAS_HOME" ] && SIWPAS_HOME=`cygpath --unix "$SIWPAS_HOME"`
  [ -n "$SIWPAS_BASE" ] && SIWPAS_BASE=`cygpath --unix "$SIWPAS_BASE"`
  [ -n "$CLASSPATH" ] && CLASSPATH=`cygpath --path --unix "$CLASSPATH"`
fi

# For OS400
if $os400; then
  # Set job priority to standard for interactive (interactive - 6) by using
  # the interactive priority - 6, the helper threads that respond to requests
  # will be running at the same priority as interactive jobs.
  COMMAND='chgjob job('$JOBNAME') runpty(6)'
  system $COMMAND

  # Enable multi threading
  export QIBM_MULTI_THREADED=Y
fi

# Get standard Java environment variables
if $os400; then
  # -r will Only work on the os400 if the files are:
  # 1. owned by the user
  # 2. owned by the PRIMARY group of the user
  # this will not work if the user belongs in secondary groups
  BASEDIR="$SIWPAS_HOME"
  . "$SIWPAS_HOME"/bin/setclasspath.sh
else
  if [ -r "$SIWPAS_HOME"/bin/setclasspath.sh ]; then
    BASEDIR="$SIWPAS_HOME"
    . "$SIWPAS_HOME"/bin/setclasspath.sh
  else
    echo "Cannot find $SIWPAS_HOME/bin/setclasspath.sh"
    echo "This file is needed to run this program"
    exit 1
  fi
fi

# Add on extra jar files to CLASSPATH
if [ ! -z "$CLASSPATH" ] ; then
  CLASSPATH="$CLASSPATH":
fi
CLASSPATH="$CLASSPATH""$SIWPAS_HOME"/bin/bootstrap.jar:"$SIWPAS_HOME"/bin/license.jar:"$SIWPAS_HOME"/bin/bcastlegpg.jar:"$SIWPAS_HOME"/bin/bcastleprov.jar

if [ -z "$SIWPAS_BASE" ] ; then
  SIWPAS_BASE="$SIWPAS_HOME"
fi

if [ -z "$SIWPAS_OUT" ] ; then
  SIWPAS_OUT="$SIWPAS_BASE"/logs/siwpas.out
fi

if [ -z "$SIWPAS_TMPDIR" ] ; then
  # Define the java.io.tmpdir to use for Siwpas
  SIWPAS_TMPDIR="$SIWPAS_BASE"/temp
fi

# Add tomcat-juli.jar to classpath
# tomcat-juli.jar can be over-ridden per instance
if [ -r "$SIWPAS_BASE/bin/tomcat-juli.jar" ] ; then
  CLASSPATH=$CLASSPATH:$SIWPAS_BASE/bin/tomcat-juli.jar
else
  CLASSPATH=$CLASSPATH:$SIWPAS_HOME/bin/tomcat-juli.jar
fi

# Bugzilla 37848: When no TTY is available, don't output to console
have_tty=0
if [ "`tty`" != "not a tty" ]; then
    have_tty=1
fi

# For Cygwin, switch paths to Windows format before running java
if $cygwin; then
  JAVA_HOME=`cygpath --absolute --windows "$JAVA_HOME"`
  JRE_HOME=`cygpath --absolute --windows "$JRE_HOME"`
  SIWPAS_HOME=`cygpath --absolute --windows "$SIWPAS_HOME"`
  SIWPAS_BASE=`cygpath --absolute --windows "$SIWPAS_BASE"`
  SIWPAS_TMPDIR=`cygpath --absolute --windows "$SIWPAS_TMPDIR"`
  CLASSPATH=`cygpath --path --windows "$CLASSPATH"`
  JAVA_ENDORSED_DIRS=`cygpath --path --windows "$JAVA_ENDORSED_DIRS"`
fi

# Set juli LogManager config file if it is present and an override has not been issued
if [ -z "$LOGGING_CONFIG" ]; then
  if [ -r "$SIWPAS_BASE"/conf/logging.properties ]; then
    LOGGING_CONFIG="-Djava.util.logging.config.file=$SIWPAS_BASE/conf/logging.properties"
  else
    # Bugzilla 45585
    LOGGING_CONFIG="-Dnop"
  fi
fi

# Set JAAS config file if it is present and an override has not been issued
if [ -z "$JAAS_CONFIG" ]; then
  if [ -r "$SIWPAS_BASE"/conf/login.config ]; then
    JAAS_CONFIG="-Djava.security.auth.login.config=$SIWPAS_BASE/conf/login.config"
  else
    # Bugzilla 45585
    JAAS_CONFIG="-Dnop"
  fi
fi

if [ -z "$LOGGING_MANAGER" ]; then
  JAVA_OPTS="$JAVA_OPTS -Djava.util.logging.manager=org.apache.juli.ClassLoaderLogManager"
else
  JAVA_OPTS="$JAVA_OPTS $LOGGING_MANAGER"
fi

# Uncomment the following line to make the umask available when using the
# org.apache.catalina.security.SecurityListener
#JAVA_OPTS="$JAVA_OPTS -Dorg.apache.catalina.security.SecurityListener.UMASK=`umask`"

# Add OpenEJB javaagent
if [ -r "$SIWPAS_HOME"/lib/openejb-agent.jar ]; then
  JAVA_OPTS="$JAVA_OPTS -javaagent:$SIWPAS_HOME/lib/openejb-agent.jar=enabled=false,debug=false"
fi

# ----- Execute The Requested Command -----------------------------------------

# Bugzilla 37848: only output this if we have a TTY
if [ $have_tty -eq 1 ]; then
  echo "Using SIWPAS_BASE:   $SIWPAS_BASE"
  echo "Using SIWPAS_HOME:   $SIWPAS_HOME"
  echo "Using SIWPAS_TMPDIR: $SIWPAS_TMPDIR"
  if [ "$1" = "debug" ] ; then
    echo "Using JAVA_HOME:       $JAVA_HOME"
  else
    echo "Using JRE_HOME:        $JRE_HOME"
  fi
  echo "Using CLASSPATH:       $CLASSPATH"
  if [ ! -z "$SIWPAS_PID" ]; then
    echo "Using SIWPAS_PID:    $SIWPAS_PID"
  fi
fi

if [ "$1" = "jpda" ] ; then
  if [ -z "$JPDA_TRANSPORT" ]; then
    JPDA_TRANSPORT="dt_socket"
  fi
  if [ -z "$JPDA_ADDRESS" ]; then
    JPDA_ADDRESS="8000"
  fi
  if [ -z "$JPDA_SUSPEND" ]; then
    JPDA_SUSPEND="n"
  fi
  if [ -z "$JPDA_OPTS" ]; then
    JPDA_OPTS="-agentlib:jdwp=transport=$JPDA_TRANSPORT,address=$JPDA_ADDRESS,server=y,suspend=$JPDA_SUSPEND"
  fi
  SIWPAS_OPTS="$SIWPAS_OPTS $JPDA_OPTS"
  shift
fi

if [ "$1" = "debug" ] ; then
  if $os400; then
    echo "Debug command not available on OS400"
    exit 1
  else
    shift
    if [ "$1" = "-security" ] ; then
      if [ $have_tty -eq 1 ]; then
        echo "Using Security Manager"
      fi
      shift
      exec "$_RUNJDB" "$JAAS_CONFIG" "$LOGGING_CONFIG" $JAVA_OPTS $SIWPAS_OPTS \
        -Djava.endorsed.dirs="$JAVA_ENDORSED_DIRS" -classpath "$CLASSPATH" \
        -sourcepath "$SIWPAS_HOME"/../../java \
        -Djava.security.manager \
        -Djava.security.policy=="$SIWPAS_BASE"/conf/catalina.policy \
        -Dcatalina.base="$SIWPAS_BASE" \
        -Dcatalina.home="$SIWPAS_HOME" \
        -Djava.io.tmpdir="$SIWPAS_TMPDIR" \
        org.apache.catalina.startup.Bootstrap "$@" start
    else
      exec "$_RUNJDB" "$JAAS_CONFIG" "$LOGGING_CONFIG" $JAVA_OPTS $SIWPAS_OPTS \
        -Djava.endorsed.dirs="$JAVA_ENDORSED_DIRS" -classpath "$CLASSPATH" \
        -sourcepath "$SIWPAS_HOME"/../../java \
        -Dcatalina.base="$SIWPAS_BASE" \
        -Dcatalina.home="$SIWPAS_HOME" \
        -Djava.io.tmpdir="$SIWPAS_TMPDIR" \
        org.apache.catalina.startup.Bootstrap "$@" start
    fi
  fi

elif [ "$1" = "run" ]; then

  shift
  if [ "$1" = "-security" ] ; then
    if [ $have_tty -eq 1 ]; then
      echo "Using Security Manager"
    fi
    shift
    eval \"$_RUNJAVA\" \"$JAAS_CONFIG\" \"$LOGGING_CONFIG\" $JAVA_OPTS $SIWPAS_OPTS \
      -Djava.endorsed.dirs=\"$JAVA_ENDORSED_DIRS\" -classpath \"$CLASSPATH\" \
      -Djava.security.manager \
      -Djava.security.policy==\"$SIWPAS_BASE/conf/catalina.policy\" \
      -Dcatalina.base=\"$SIWPAS_BASE\" \
      -Dcatalina.home=\"$SIWPAS_HOME\" \
      -Djava.io.tmpdir=\"$SIWPAS_TMPDIR\" \
      org.apache.catalina.startup.Bootstrap "$@" start
  else
    eval \"$_RUNJAVA\" \"$JAAS_CONFIG\" \"$LOGGING_CONFIG\" $JAVA_OPTS $SIWPAS_OPTS \
      -Djava.endorsed.dirs=\"$JAVA_ENDORSED_DIRS\" -classpath \"$CLASSPATH\" \
      -Dcatalina.base=\"$SIWPAS_BASE\" \
      -Dcatalina.home=\"$SIWPAS_HOME\" \
      -Djava.io.tmpdir=\"$SIWPAS_TMPDIR\" \
      org.apache.catalina.startup.Bootstrap "$@" start
  fi

elif [ "$1" = "start" ] ; then

  if [ ! -z "$SIWPAS_PID" ]; then
    if [ -f "$SIWPAS_PID" ]; then
      if [ -s "$SIWPAS_PID" ]; then
        echo "Existing PID file found during start."
        if [ -r "$SIWPAS_PID" ]; then
          PID=`cat "$SIWPAS_PID"`
          ps -p $PID >/dev/null 2>&1
          if [ $? -eq 0 ] ; then
            echo "Siwpas appears to still be running with PID $PID. Start aborted."
            exit 1
          else
            echo "Removing/clearing stale PID file."
            rm -f "$SIWPAS_PID" >/dev/null 2>&1
            if [ $? != 0 ]; then
              if [ -w "$SIWPAS_PID" ]; then
                cat /dev/null > "$SIWPAS_PID"
              else
                echo "Unable to remove or clear stale PID file. Start aborted."
                exit 1
              fi
            fi
          fi
        else
          echo "Unable to read PID file. Start aborted."
          exit 1
        fi
      else
        rm -f "$SIWPAS_PID" >/dev/null 2>&1
        if [ $? != 0 ]; then
          if [ ! -w "$SIWPAS_PID" ]; then
            echo "Unable to remove or write to empty PID file. Start aborted."
            exit 1
          fi
        fi
      fi
    fi
  fi

  shift
  touch "$SIWPAS_OUT"
  if [ "$1" = "-security" ] ; then
    if [ $have_tty -eq 1 ]; then
      echo "Using Security Manager"
    fi
    shift
    eval \"$_RUNJAVA\" \"$JAAS_CONFIG\" \"$LOGGING_CONFIG\" $JAVA_OPTS $SIWPAS_OPTS \
      -Djava.endorsed.dirs=\"$JAVA_ENDORSED_DIRS\" -classpath \"$CLASSPATH\" \
      -Djava.security.manager \
      -Djava.security.policy==\"$SIWPAS_BASE/conf/catalina.policy\" \
      -Dcatalina.base=\"$SIWPAS_BASE\" \
      -Dcatalina.home=\"$SIWPAS_HOME\" \
      -Djava.io.tmpdir=\"$SIWPAS_TMPDIR\" \
      org.apache.catalina.startup.Bootstrap "$@" start \
      >> "$SIWPAS_OUT" 2>&1 "&"

  else
    eval \"$_RUNJAVA\" \"$JAAS_CONFIG\" \"$LOGGING_CONFIG\" $JAVA_OPTS $SIWPAS_OPTS \
      -Djava.endorsed.dirs=\"$JAVA_ENDORSED_DIRS\" -classpath \"$CLASSPATH\" \
      -Dcatalina.base=\"$SIWPAS_BASE\" \
      -Dcatalina.home=\"$SIWPAS_HOME\" \
      -Djava.io.tmpdir=\"$SIWPAS_TMPDIR\" \
      org.apache.catalina.startup.Bootstrap "$@" start \
      >> "$SIWPAS_OUT" 2>&1 "&"

  fi

  if [ ! -z "$SIWPAS_PID" ]; then
    echo $! > "$SIWPAS_PID"
  fi

elif [ "$1" = "stop" ] ; then

  shift

  SLEEP=5
  if [ ! -z "$1" ]; then
    echo $1 | grep "[^0-9]" >/dev/null 2>&1
    if [ $? -gt 0 ]; then
      SLEEP=$1
      shift
    fi
  fi

  FORCE=0
  if [ "$1" = "-force" ]; then
    shift
    FORCE=1
  fi

  if [ ! -z "$SIWPAS_PID" ]; then
    if [ -s "$SIWPAS_PID" ]; then
      if [ -f "$SIWPAS_PID" ]; then
        kill -0 `cat "$SIWPAS_PID"` >/dev/null 2>&1
        if [ $? -gt 0 ]; then
          echo "PID file found but no matching process was found. Stop aborted."
          exit 1
        fi
      else
        echo "\$SIWPAS_PID was set but the specified file does not exist. Is Siwpas running? Stop aborted."
        exit 1
      fi
    else
      echo "PID file is empty and has been ignored."
    fi
  fi

  eval \"$_RUNJAVA\" $JAVA_OPTS \
    -Djava.endorsed.dirs=\"$JAVA_ENDORSED_DIRS\" -classpath \"$CLASSPATH\" \
    -Dcatalina.base=\"$SIWPAS_BASE\" \
    -Dcatalina.home=\"$SIWPAS_HOME\" \
    -Djava.io.tmpdir=\"$SIWPAS_TMPDIR\" \
    org.apache.catalina.startup.Bootstrap "$@" stop

  if [ ! -z "$SIWPAS_PID" ]; then
    if [ -f "$SIWPAS_PID" ]; then
      while [ $SLEEP -ge 0 ]; do
        kill -0 `cat "$SIWPAS_PID"` >/dev/null 2>&1
        if [ $? -gt 0 ]; then
          rm -f "$SIWPAS_PID" >/dev/null 2>&1
          if [ $? != 0 ]; then
            if [ -w "$SIWPAS_PID" ]; then
              cat /dev/null > "$SIWPAS_PID"
            else
              echo "Siwpas stopped but the PID file could not be removed or cleared."
            fi
          fi
          break
        fi
        if [ $SLEEP -gt 0 ]; then
          sleep 1
        fi
        if [ $SLEEP -eq 0 ]; then
          if [ $FORCE -eq 0 ]; then
            echo "Siwpas did not stop in time. PID file was not removed."
          fi
        fi
        SLEEP=`expr $SLEEP - 1 `
      done
    fi
  fi

  if [ $FORCE -eq 1 ]; then
    if [ -z "$SIWPAS_PID" ]; then
      echo "Kill failed: \$SIWPAS_PID not set"
    else
      if [ -f "$SIWPAS_PID" ]; then
        PID=`cat "$SIWPAS_PID"`
        echo "Killing Siwpas with the PID: $PID"
        kill -9 $PID
        rm -f "$SIWPAS_PID" >/dev/null 2>&1
        if [ $? != 0 ]; then
          echo "Siwpas was killed but the PID file could not be removed."
        fi
      fi
    fi
  fi

elif [ "$1" = "configtest" ] ; then

    eval \"$_RUNJAVA\" $JAVA_OPTS $SIWPAS_OPTS \
      -Djava.endorsed.dirs=\"$JAVA_ENDORSED_DIRS\" -classpath \"$CLASSPATH\" \
      -Dcatalina.base=\"$SIWPAS_BASE\" \
      -Dcatalina.home=\"$SIWPAS_HOME\" \
      -Djava.io.tmpdir=\"$SIWPAS_TMPDIR\" \
      org.apache.catalina.startup.Bootstrap configtest 
    exit $?

elif [ "$1" = "version" ] ; then

    "$_RUNJAVA"   \
      -classpath "$SIWPAS_HOME/lib/siwpas-loader.jar" \
      org.ow2.siwpas.Version

else

  echo "Usage: siwpas.sh ( commands ... )"
  echo "commands:"
  if $os400; then
    echo "  debug             Start Siwpas in a debugger (not available on OS400)"
    echo "  debug -security   Debug Siwpas with a security manager (not available on OS400)"
  else
    echo "  debug             Start Siwpas in a debugger"
    echo "  debug -security   Debug Siwpas with a security manager"
  fi
  echo "  jpda start        Start Siwpas under JPDA debugger"
  echo "  run               Start Siwpas in the current window"
  echo "  run -security     Start in the current window with security manager"
  echo "  start             Start Siwpas in a separate window"
  echo "  start -security   Start in a separate window with security manager"
  echo "  stop              Stop Siwpas, waiting up to 5 seconds for the process to end"
  echo "  stop n            Stop Siwpas, waiting up to n seconds for the process to end"
  echo "  stop -force       Stop Siwpas, wait up to 5 seconds and then use kill -KILL if still running"
  echo "  stop n -force     Stop Siwpas, wait up to n seconds and then use kill -KILL if still running"
  echo "  version           What version of siwpas are you running?"
  echo "Note: Waiting for the process to end and use of the -force option require that \$SIWPAS_PID is defined"
  exit 1

fi
